"""Module for the machine learning interface using Tkinter.

This module creates a window with options for selecting different 
Machine Learning algorithms and integrates functionalities such as
parameter selection, data preview, and result visualization.
"""

from tkinter import Toplevel
from .view import View
from .utils import *
from .gui_helpers import *
from ..data_processing.data_processing import DataProcessing

class MachineLearning(Toplevel):
    def __init__(self, master=None):
        Toplevel.__init__(self, master=master)
        self.data_s = StringVar()
        self.data_s.set('Target city')
        self.list_dt = ['Target city', 'Neighbor A', 'Neighbor B', 'Neighbor C']
            
        self.ind_s = StringVar()
        self.ind_s.set('Maximum temperature')
        self.list_ind = ["Precipitation", 'Maximum temperature', 'Minimum temperature']
            
        self.title('Machine learning')
        self.geometry('800x800')
        self.configure(background=fundo)

        Label(self, text='MACHINE LEARNING', font='Arial 14 bold',
               fg='white', bg=fundo).place(x=200, y=20)

        self.ml_selected = StringVar()
        self.ml_selected.set('Decision Trees')
        lista_ml = ['Decision Trees',
                    'Bagged Trees',
                     'Neural network',
                     'Nearest Neighbors',
                     'Support Vector',
                     'Gaussian Process']
        
        combo = ttk.Combobox(self, values=lista_ml, textvariable=self.ml_selected,
                              width=28, font='Arial 12', justify=CENTER, state='readonly')
        combo.place(x=20, y=60)
        Button(self, text='Choose Machine Learning', font='Arial 11 bold',
                fg='white', bg=fun_ap, width=30, command=self.generate_param).place(x=340, y=59)

    def int_float(self, value):
        """
        Try to return a int value, if it can't, returns a float value
        """
        try:
            return int(value)
        except:
            return float(value)

    def valid_maxf(self, value):
        """
        Returnes the correct type of a received value
        """
        if value.isdigit() == True:
            value = int(value)
        elif value.isalnum() == True and value.isdigit() == False:
            value = str(value)
        elif value.isalnum() == False and value.isdigit() == False and value.isalpha() == False:
            value = float(value)

        return value

    def save_parameter(self):
        v = View
        return v.save_parameter()

    def data_preview(self, score, mean_abs_error, mean_rel_error, max_abs_error, exact_max,
                     pred_max, min_abs_error, exact_min, pred_min, y_exact, y_pred, x_axis):
        v = View()
        v.data_preview(self, score, mean_abs_error, mean_rel_error, max_abs_error, exact_max,
                       pred_max, min_abs_error, exact_min, pred_min, y_exact, y_pred, x_axis)

    def get_end(self, city):
        treatment = DataProcessing()
        return treatment.get_file_path(city)

    def generate_preview_dt(self):
        v = View()
        v.generate_preview_dt(self)

    def generate_preview_bt(self):
        v = View()
        v.generate_preview_bt(self)

    def generate_preview_nn(self):
        v = View()
        v.generate_preview_nn(self)

    def generate_preview_svm(self):
        v = View()
        v.generate_preview_svm(self)

    def generate_preview_kn(self):
        v = View()
        v.generate_preview_kn(self)

    def generate_param(self):
        opcao = self.ml_selected.get()
        if opcao == 'Decision Trees':
            Canvas(self, width=615, height=900, background=fundo, border=0).place(x=10, y=95)
            self.param_frame = DTParameterFrame(self, fundo, fun_alt)
            
            Label(self, text="Training data:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=520)
            self.combo_c = ttk.Combobox(self, values=self.list_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=545)

            Label(self, text='Indicador:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=520)
            ttk.Combobox(self, values=self.list_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=545)

            self.por_trei = IntVar()
            self.por_trei.set(70)
            Label(self, text="Training portion:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=580)
            Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=605)
        
            self.num_teste = IntVar()
            self.num_teste.set(5)
            Label(self, text="Number of tests (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=580)
            self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=605)
           
            Button(self, text='Preview', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.generate_preview_dt).place(x=50, y=685)
            #Button(self, text='Salvar Paramt.', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.salvar_paramt).place(x=340, y=685)
            self.save_model = IntVar()
            Checkbutton(self, text='Save model', variable=self.save_model, bg=fundo, font='Arial 12 bold', activebackground=fundo).place(x=340, y=685)
        elif opcao == 'Bagged Trees':
            Canvas(self, width=615, height=900, background=fundo, border=0).place(x=10, y=95)
            self.param_frame = DTParameterFrame(self, fundo, fun_alt)
            
            Label(self, text="Training data:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=520)
            self.combo_c = ttk.Combobox(self, values=self.list_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=545)

            Label(self, text='Indicador:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=520)
            ttk.Combobox(self, values=self.list_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=545)

            self.por_trei = IntVar()
            self.por_trei.set(70)
            Label(self, text="Training portion:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=580)
            Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=605)
        
            self.num_teste = IntVar()
            self.num_teste.set(5)
            Label(self, text="Number of tests (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=580)
            self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=605)

            self.n_estimators = IntVar()
            self.n_estimators.set(10)
            Label(self, text='Number of estimators (int):', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=640)
            self.ent_n_estimators = Entry(self, textvariable=self.n_estimators, width=27, font='Arial 12', justify=CENTER).place(x=50, y=665)

            Button(self, text='Preview', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.generate_preview_bt).place(x=50, y=685)
            #Button(self, text='Salvar Paramt.', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.salvar_paramt).place(x=340, y=685)
            self.save_model = IntVar()
            Checkbutton(self, text='Save model', variable=self.save_model, bg=fundo, font='Arial 12 bold', activebackground=fundo).place(x=340, y=685)
        
        elif opcao == 'Neural network':
            Canvas(self, width=615, height=900, background=fundo, border=0).place(x=10, y=95)
            self.param_frame = NNParameterFrame(self, fundo)
            Label(self, text="Training data:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=750)
            self.combo_c = ttk.Combobox(self, values=self.list_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=775)

            Label(self, text='Indicator:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=750)
            ttk.Combobox(self, values=self.list_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=775)

            self.por_trei = IntVar()
            self.por_trei.set(70)
            Label(self, text="Training portion:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=810)
            Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=835)
        
            self.num_teste = IntVar()
            self.num_teste.set(5)
            Label(self, text="Number of tests (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=810)
            self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=835)

            Button(self, text='Preview', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.generate_preview_nn).place(x=50, y=915)
            self.save_model = IntVar()
            Checkbutton(self, text='Save model', variable=self.save_model, bg=fundo, font='Arial 12 bold', activebackground=fundo).place(x=340, y=915)
        elif opcao == 'Nearest Neighbors':
           Canvas(self, width=615, height=900, background=fundo, border=0).place(x=10, y=95)
           self.param_frame = NNeighParameterFrame(self, fundo)

           self.lbf_para_nn = LabelFrame(self, text='Parameters', width=600, height=205, font='Arial 12 bold', fg='white', bg=fundo).place(x=20, y=100) 

           Label(self, text="Training data:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=340)
           self.combo_c = ttk.Combobox(self, values=self.list_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=365)

           Label(self, text='Indicator:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=340)
           ttk.Combobox(self, values=self.list_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=365)

           self.por_trei = IntVar()
           self.por_trei.set(70)
           Label(self, text="Training portion:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=400)
           Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=425)

           self.num_teste = IntVar()
           self.num_teste.set(5)
           Label(self, text="Number of tests (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=400)
           self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=425)

           Button(self, text='Preview', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.generate_preview_kn).place(x=50, y=505)
           self.save_model = IntVar()
           Checkbutton(self, text='Save model', variable=self.save_model, bg=fundo, font='Arial 12 bold', activebackground=fundo).place(x=340, y=505) 
        elif opcao == 'Support Vector':
            w = Canvas(self, width=615, height=900, background=fundo, border=0)
            w.place(x=10, y=95)
            self.lbf_para_nn = LabelFrame(self, text='Parâmetros', width=600, height=385, font='Arial 12 bold', fg='white', bg=fundo).place(x=20, y=100)
            
            self.kernel_v = StringVar()
            lista_ker = ['linear', 'poly', 'rbf', 'sigmoid']
            self.kernel_v.set('rbf')
            Label(self, text='Kernel:', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=120)
            ttk.Combobox(self, values=lista_ker, textvariable=self.kernel_v, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=145)

            self.degree_v = IntVar()
            self.degree_v.set(3)
            Label(self, text='Degree (int):', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=120)
            Entry(self, textvariable=self.degree_v, font='Arial 12', width=27, justify=CENTER).place(x=340, y=145) 

            self.gamma_v = StringVar()
            self.gamma_v.set('scale')
            Label(self, text='Gamma ("scale", "auto", float):', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=180)
            Entry(self, textvariable=self.gamma_v, font='Arial 12', width=27, justify=CENTER).place(x=50, y=205)

            self.coef0_v = StringVar()
            self.coef0_v.set('0.0')
            Label(self, text='Coef0 (float):', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=180)
            Entry(self, textvariable=self.coef0_v, font='Arial 12', width=27, justify=CENTER).place(x=340, y=205)

            self.tol_v = StringVar()
            self.tol_v.set('0.001')
            Label(self, text='Tol (float):', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=240)
            Entry(self, textvariable=self.tol_v, font='Arial 12', width=27, justify=CENTER).place(x=50, y=265)

            self.c_v = StringVar()
            self.c_v.set('1.0')
            Label(self, text='C (float):', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=240)
            Entry(self, textvariable=self.c_v, font='Arial 12', width=27, justify=CENTER).place(x=340, y=265)

            self.epsilon_v = StringVar()
            self.epsilon_v.set('0.1')
            Label(self, text='Epsilon (float):', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=300)
            Entry(self, textvariable=self.epsilon_v, font='Arial 12', width=27, justify=CENTER).place(x=50, y=325)   

            self.shrinking_v = BooleanVar()
            self.shrinking_v.set(True)
            Label(self, text='Shrinking (Bool):', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=300)
            Entry(self, textvariable=self.shrinking_v, font='Arial 12', width=27, justify=CENTER).place(x=340, y=325)

            self.cache_size_v = StringVar()
            self.cache_size_v.set('200')
            Label(self, text='Cache_size (float):', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=360)
            Entry(self, textvariable=self.cache_size_v, font='Arial 12', width=27, justify=CENTER).place(x=50, y=385)   

            self.verbose_v = BooleanVar()
            self.verbose_v.set(False)
            Label(self, text='Verbose (Bool):', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=360)
            Entry(self, textvariable=self.verbose_v, font='Arial 12', width=27, justify=CENTER).place(x=340, y=385)

            self.maxiter_v = IntVar()
            self.maxiter_v.set(-1)
            Label(self, text='Max_iter (int):', font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=420)
            Entry(self, textvariable=self.maxiter_v, font='Arial 12', width=27, justify=CENTER).place(x=50, y=445)

            self.lbf_dt_nn = LabelFrame(self, text='Data', width=600, height=170, font='Arial 12 bold', fg ='white', bg=fundo).place(x=20, y=500)

            Label(self, text="Training data:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=520)
            self.combo_c = ttk.Combobox(self, values=self.list_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=545)

            Label(self, text='Indicator:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=520)
            ttk.Combobox(self, values=self.list_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=545)

            self.por_trei = IntVar()
            self.por_trei.set(70)
            Label(self, text="Training portion:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=580)
            Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=605)
        
            self.num_teste = IntVar()
            self.num_teste.set(5)
            Label(self, text="Number of tests (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=580)
            self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=605)

            Button(self, text='Preview', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.generate_preview_svm).place(x=50, y=680)
            self.save_model = IntVar()
            Checkbutton(self, text='Save model', variable=self.save_model, bg=fundo, font='Arial 12 bold', activebackground=fundo).place(x=340, y=680)
        elif opcao == 'Gaussian Process':
            w = Canvas(self, width=615, height=900, background=fundo, border=0)
            w.place(x=10, y=95)
            self.param_frame = GPParameterFrame(self, fundo)
            
            Label(self, text="Training data:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=340)
            self.combo_c = ttk.Combobox(self, values=self.list_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=365)

            Label(self, text='Indicator:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=340)
            ttk.Combobox(self, values=self.list_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=365)

            self.por_trei = IntVar()
            self.por_trei.set(70)
            Label(self, text="Training portion:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=400)
            Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=425)
        
            self.num_teste = IntVar()
            self.num_teste.set(5)
            Label(self, text="Number of tests (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=400)
            self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=425)

            Button(self, text='Preview', font='Arial 11 bold', fg='white', bg=fun_b, width=25, command=self.generate_preview_svm).place(x=50, y=505)
            self.save_model = IntVar()
            Checkbutton(self, text='Save model', variable=self.save_model, bg=fundo, font='Arial 12 bold', activebackground=fundo).place(x=340, y=505)



from ..utils import *
if typing.TYPE_CHECKING:
    from .data_imputation_page import DataImputationPage
from .triangulation_page import TriangulationPage
from .machine_learning.machine_learning_page import MachineLearningPage
from .meta_learning.meta_learning_page import MetaLearningPage
from ..tooltip import CreateToolTip

class ImputationTechniquesPage(ttk.Frame):
    """Tela para escolher a técnica de imputação."""
    def __init__(self, parent, controller):
        ttk.Frame.__init__(self, parent)
        self.controller = controller
        i18n = controller.i18n

        # --- Frame Superior ---
        top_frame = ttk.Frame(self)
        top_frame.pack(fill=tk.X, padx=10, pady=10)

        self.back_button = ttk.Button(top_frame, text="", command=self.go_to_data_imputation)
        self.back_button.pack(side=tk.LEFT)

        self.page_title = ttk.Label(top_frame, text="", font=("Verdana", 16, "bold"))
        self.page_title.pack(side=tk.LEFT, expand=True)

        main_container = ttk.Frame(self)
        main_container.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)
        main_container.grid_columnconfigure(0, weight=1, minsize=250)
        main_container.grid_columnconfigure(1, weight=3)
        main_container.grid_rowconfigure(0, weight=1)
        
        # --- PAINEL DA ESQUERDA ---
        left_panel = ttk.Frame(main_container)
        left_panel.grid(row=0, column=0, sticky="nsew", padx=(0, 10))
        
        self.choose_technique_label = ttk.Label(left_panel, text="", font=("Verdana", 12))
        self.choose_technique_label.pack(anchor="w", pady=(0, 10), padx=5)

        self.triangulation_btn = ttk.Button(left_panel, text="", command=lambda: controller.show_frame(TriangulationPage))
        self.triangulation_btn.pack(fill=tk.X, pady=5)
        self.triangulation_hint = CreateToolTip(self.triangulation_btn, text=i18n.get('triangulation_input_hint'))

        self.machine_learning_btn = ttk.Button(left_panel, text="", command=lambda: controller.show_frame(MachineLearningPage))
        self.machine_learning_btn.pack(fill=tk.X, pady=5)
        self.machine_learning_hint = CreateToolTip(self.machine_learning_btn, text=i18n.get('machine_learning_input_hint'))
        
        self.meta_learning_btn = ttk.Button(left_panel, text="", command=lambda: controller.show_frame(MetaLearningPage))
        self.meta_learning_btn.pack(fill=tk.X, pady=5)
        self.meta_learning_hint = CreateToolTip(self.meta_learning_btn, text=i18n.get('meta_learning_input_hint'))


        # --- PAINEL DA DIREITA ---
        right_panel = ttk.Frame(main_container, relief="solid", borderwidth=1)
        right_panel.grid(row=0, column=1, sticky="nsew")
        
        self.update_texts()

    def go_to_data_imputation(self):
        """Importa e navega para a página inicial."""
        from .data_imputation_page  import DataImputationPage
        self.controller.show_frame(DataImputationPage)
        
    def update_texts(self):
        """Atualiza os textos APENAS para esta tela."""
        i18n = self.controller.i18n
        
        self.back_button.config(text=i18n.get('back_btn'))
        self.page_title.config(text=i18n.get('imputation_techniques_page_title'))
        self.choose_technique_label.config(text=i18n.get('choose_a_technique_label'))
        self.triangulation_btn.config(text=i18n.get('triangulation_btn'))
        self.machine_learning_btn.config(text=i18n.get('machine_learning_btn'))
        self.meta_learning_btn.config(text=i18n.get('meta_learning_btn'))
        self.triangulation_hint.text = i18n.get('triangulation_input_hint')
        self.machine_learning_hint.text = i18n.get('machine_learning_input_hint')
        self.meta_learning_hint.text = i18n.get('meta_learning_input_hint')



from ..utils import *
from ..tooltip import CreateToolTip

if typing.TYPE_CHECKING:
    from .imputation_techniques_page import ImputationTechniquesPage

class TriangulationPage(ttk.Frame):
    """Tela para configurar e executar a imputação por triangulação."""
    def __init__(self, parent, controller):
        ttk.Frame.__init__(self, parent)
        self.controller = controller
        i18n = controller.i18n

        self.graph_canvas = None
        self.toolbar = None

        top_frame = ttk.Frame(self)
        top_frame.pack(fill=tk.X, padx=10, pady=10)

        self.back_button = ttk.Button(top_frame, text="", command=self.go_to_imputation_techniques)
        self.back_button.pack(side=tk.LEFT)

        self.page_title = ttk.Label(top_frame, text="", font=("Verdana", 16, "bold"))
        self.page_title.pack(side=tk.LEFT, expand=True)

        # --- Layout Principal de Duas Colunas ---
        main_container = ttk.Frame(self)
        main_container.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)
        main_container.grid_columnconfigure(0, weight=1, minsize=250)
        main_container.grid_columnconfigure(1, weight=3)
        main_container.grid_rowconfigure(0, weight=1)
        
        # --- PAINEL DA ESQUERDA ---
        left_panel = ttk.Frame(main_container)
        left_panel.grid(row=0, column=0, sticky="nsew", padx=(0, 10))
        
        self.method_label = ttk.Label(left_panel, text="")
        self.method_label.pack(anchor="w", pady=(0, 2), padx=5)
        
        self.method_combo = ttk.Combobox(left_panel, state="readonly", 
                                         values=['Arithmetic Average', 'Inverse Distance Weighted', 
                                                 'Regional Weight', 'Optimized Normal Ratio'])
        self.method_combo.pack(fill=tk.X, pady=(0, 15), padx=5)
        self.method_hint = CreateToolTip(self.method_combo, text=i18n.get('method_triang_hint'))

        self.parameter_label = ttk.Label(left_panel, text="")
        self.parameter_label.pack(anchor="w", pady=(0, 2), padx=5)

        self.parameter_combo = ttk.Combobox(left_panel, state="readonly",
                                            values=["Precipitation", 'Maximum temperature', "Minimum temperature"])
        self.parameter_combo.pack(fill=tk.X, pady=(0, 15), padx=5)
        self.param_hint = CreateToolTip(self.parameter_combo, text=i18n.get('param_triang_hint'))

        self.run_triangulation_btn = ttk.Button(left_panel, text="", command=self.run_triangulation_analysis)
        self.run_triangulation_btn.pack(pady=20, padx=5)
        self.run_btn_hint = CreateToolTip(self.run_triangulation_btn, text=i18n.get('btn_triang_hint'))


        # --- PAINEL DA DIREITA ---
        self.right_panel = ttk.Frame(main_container, relief="solid", borderwidth=1)
        self.right_panel.grid(row=0, column=1, sticky="nsew")
        
        self.update_texts()

    def go_to_imputation_techniques(self):
        """Importa e navega para a página inicial."""
        from .imputation_techniques_page import ImputationTechniquesPage
        self.controller.show_frame(ImputationTechniquesPage)

    def run_triangulation_analysis(self):
        """Pega os dados dos combos, executa a triangulação e plota o gráfico."""
        if self.graph_canvas:
            self.graph_canvas.get_tk_widget().destroy()
        if self.toolbar:
            self.toolbar.destroy()

        met = self.method_combo.get()
        ind = self.parameter_combo.get()

        if not met:
            self.controller.show_translated_message(
                msg_type='warning',
                title_key='missing_method_title',
                message_key='select_a_triang_method_msg'
                )
            return
        if not ind:
            self.controller.show_translated_message(
                msg_type='warning',
                title_key='missing_parameter_title',
                message_key='select_a_parameter_msg'
                )
            return

        trian = Triangulation() 

        if ind == "Precipitation":
            focus = 1
            y_label = "Precipitation (mm)"
        elif ind == 'Maximum temperature':
            focus = 2
            y_label = "Temperature(°C)"
        else: # Minimum temperature
            focus = 3
            y_label = "Temperature(°C)"

        if met == 'Arithmetic Average':
            trian.avg(focus)
            eixo_x, eixo_y_tri, eixo_y_exato, media_ea, media_er, _ = trian.get_avg()
        elif met == 'Inverse Distance Weighted':
            trian.idw(focus)
            eixo_x, eixo_y_tri, eixo_y_exato, media_ea, media_er, _ = trian.get_idw()
        elif met == 'Regional Weight':
            trian.rw(focus)
            eixo_x, eixo_y_tri, eixo_y_exato, media_ea, media_er, _ = trian.get_rw()
        else: # Optimized Normal Ratio
            trian.onr(focus)
            eixo_x, eixo_y_tri, eixo_y_exato, media_ea, media_er, _ = trian.get_onr()

        # 4. Criação e Plotagem da Figura Matplotlib
        media_ea = round(media_ea, 4)
        media_er = round(media_er, 4)
        texto_titulo = f'Mean Absolute Error: {media_ea} | Mean Relative Error: {media_er}'
        
        figura = Figure(figsize=(8, 6), dpi=100)
        plot = figura.add_subplot(111)
        plot.plot(eixo_x, eixo_y_exato, label='Exact', color='green')
        plot.plot(eixo_x, eixo_y_tri, label=f'Triangulated ({met})', color='red')
        plot.legend()
        plot.grid(True)
        plot.set_ylabel(y_label)
        plot.set_xlabel("Comparisons")
        plot.set_title(texto_titulo, fontsize=10)
        figura.tight_layout()

        self.graph_canvas = FigureCanvasTkAgg(figura, master=self.right_panel)
        self.graph_canvas.draw()
        self.graph_canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=True)

        self.toolbar = NavigationToolbar2Tk(self.graph_canvas, self.right_panel)
        self.toolbar.update()
        self.toolbar.pack(side=tk.BOTTOM, fill=tk.X)
        
    def update_texts(self):
        """Atualiza os textos APENAS para esta tela."""
        i18n = self.controller.i18n
        
        self.back_button.config(text=i18n.get('back_btn'))
        self.page_title.config(text=i18n.get('triangulation_page_title'))
        self.method_label.config(text=i18n.get('triangulation_method_label'))
        self.parameter_label.config(text=i18n.get('climatic_parameter_label'))
        self.run_triangulation_btn.config(text=i18n.get('run_triangulation_btn'))
        self.method_hint.text = i18n.get('method_triang_hint')
        self.param_hint.text = i18n.get('param_triang_hint')
        self.run_btn_hint.text = i18n.get('btn_triang_hint')


import tkinter as tk
from tkinter import ttk
from tkinter import Canvas, LabelFrame, Label, Entry, StringVar, BooleanVar, IntVar, Scale, Button, Checkbutton, HORIZONTAL, CENTER

fundo = '#4F4F4F' #? Cor de fundo da tela
fun_b = '#3CB371' #? Cor de fundo dos botoes
fun_ap = '#9C444C'
fun_alt = '#C99418'
fun_meta_le = '#191970'

def create_training_widgets(self, y_base=520):
    self.data_s = StringVar()
    self.data_s.set('Target city')
    lista_dt = ['Target city', 'Neighbor A', 'Neighbor B', 'Neighbor C']
    Label(self, text="Dados para treinamento:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=y_base)
    ttk.Combobox(self, values=lista_dt, textvariable=self.data_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=50, y=y_base + 25)

    self.ind_s = StringVar()
    self.ind_s.set('Maximum temperature')
    lista_ind = ["Precipitation", 'Maximum temperature', 'Minimum temperature']
    Label(self, text='Indicador:', font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=y_base)
    ttk.Combobox(self, values=lista_ind, textvariable=self.ind_s, width=25, font='Arial 12', justify=CENTER, state='readonly').place(x=340, y=y_base + 25)

    self.por_trei = IntVar()
    self.por_trei.set(70)
    Label(self, text="Porção para treinamento:", font='Arial 12 bold', fg='white', bg=fundo).place(x=50, y=y_base + 60)
    Scale(self, variable=self.por_trei, orient=HORIZONTAL, length=240).place(x=50, y=y_base + 85)

    self.num_teste = IntVar()
    self.num_teste.set(5)

    Label(self, text="Número de testes (int):", font='Arial 12 bold', fg='white', bg=fundo).place(x=340, y=y_base + 60)
    self.ent_num_teste = Entry(self, textvariable=self.num_teste, width=27, font='Arial 12', justify=CENTER).place(x=340, y=y_base + 85)